/**
 * The goal of this script is to control a LED on pin 5, 6, 7
 * and get temperature on pin 12.
 * 
 * Connect the ardunio to the Serial <-> USB device.
 * 
 * Connect the RGB led:
 * - Connect red to pin 5
 * - Connect green to pin 6
 * - Connect blue to pin 7
 * - Connect V to VCC
 * 
 * Connect temperature sensor to:
 * - S to pin 12
 * - Middle pin to VCC
 * - Pin "-" to GND
 * 
 * Burn the firmware to the device.
 * 
 * How to communicate with the serial port?
 * r0 to r9 : Set the power of the red light
 * g0 to g9 : Set the power of the green light
 * b0 to b9 : Set the power of the blue light
 * t : Get the temperature
 * i : Get some information
 * h : Show some help
 */

#include <Arduino.h>
#include <HardwareSerial.h>
#include <Print.h>

#include "OneWire.h"

// RGB Pins
int ledR = 5;
int ledG = 6;
int ledB = 7;

// Setup onewire
OneWire ds(12);

// Value of the r,g,b leds between 0 and 1
int cr = 0;
int cg = 0;
int cb = 0;

// ------------------------
int cnt = 0;

void setup() {

	// Start serial port
	Serial.begin(9600);

	// initialize RGB leds
	pinMode(ledR, OUTPUT);
	pinMode(ledG, OUTPUT);
	pinMode(ledB, OUTPUT);

}

/**
 * Just a "welcome" light show. To verify that the light is working as expected.
 */
void welcomeLightshow() {
	digitalWrite(ledR, LOW);
	digitalWrite(ledG, HIGH);
	digitalWrite(ledB, HIGH);
	delay(300);
	digitalWrite(ledR, HIGH);
	digitalWrite(ledG, LOW);
	digitalWrite(ledB, HIGH);
	delay(300);
	digitalWrite(ledR, HIGH);
	digitalWrite(ledG, HIGH);
	digitalWrite(ledB, LOW);
	delay(300);

}

/**
 * When you send h on the serial port
 */
void help() {
	Serial.println("r0 to r9 : Set the power of the red light");
	Serial.println("g0 to g9 : Set the power of the green light");
	Serial.println("b0 to b9 : Set the power of the blue light");
	Serial.println("t : Get the temperature");
	Serial.println("i : Get some information (power R,G,B)");
	Serial.println("h : Show some help");
}

/**
 * when you send i on the serial port
 */
void info() {
	Serial.print("r=");
	Serial.print(cr);
	Serial.print(",g=");
	Serial.print(cg);
	Serial.print(",b=");
	Serial.print(cb);
	Serial.print("\n");
}

// Address of the onewire temperature sensor
byte addr[8];
// If we know the address, don't ask it again
boolean dsFound = false;

/**
 * Set the power of the leds, PWD
 */
void leds() {
	// Set the power of the leds (PWM)
	for (int l = 0; l < 100; l++) {
		if (l%9 < cr)
			digitalWrite(ledR, LOW);
		else
			digitalWrite(ledR, HIGH);
		if (l%9 < cg)
			digitalWrite(ledG, LOW);
		else
			digitalWrite(ledG, HIGH);
		if (l%9 < cb)
			digitalWrite(ledB, LOW);
		else
			digitalWrite(ledB, HIGH);
	}
}

void temperature() {
	
	byte data[12];
	int i;
	byte present;
	int temp;
	float temp2;
	
	digitalWrite(ledR, HIGH);
	digitalWrite(ledG, HIGH);
	digitalWrite(ledB, HIGH);
	
	if (!dsFound) {
		if (!ds.search(addr)) {
			Serial.print("e:No Temperature Sensor!\n");
			ds.reset_search();
			return;
		}
		if (OneWire::crc8(addr, 7) != addr[7]) {
			Serial.print("e:CRC is not valid!\n");
			return;
		}
		if (addr[0] != 0x28) {
			Serial.print("e:Device is not a DS18S20 family device.\n");
			return;
		}
		dsFound = true;
	}

	ds.reset();
	ds.select(addr);
	ds.write(0x44, 1);    // start conversion, with parasite power on at the end
	leds();
	//delay(1000);    // maybe 750ms is enough, maybe not

	present = ds.reset();
	ds.select(addr);
	ds.write(0xBE);   // Read Scratchpad
	for (i = 0; i < 9; i++) {        // we need 9 bytes
		data[i] = ds.read();
	}
	temp = (data[1] << 8) + data[0]; //take the two bytes from the response relating to temperature
	temp2 = temp;
	temp2 = temp2 / 16;
	// Temp=Temp>>4;//divide by 16 to get pure celcius readout
	//next line is Fahrenheit conversion
	//Temp=Temp*1.8+32; // comment this line out to get celcius
	Serial.println(temp2, DEC);

}

// Last command
char command = 0;

// the loop routine runs over and over again forever:
void loop() {

	welcomeLightshow();

	while (true) {

		// Process what is received on the serial port
		int r = Serial.read();
		if (r != -1) {
			int x;
			switch (r) {
			case 'h':
				help();
				break;
			case 'r':
			case 'g':
			case 'b':
				command = (char) r;
				break;
			case 'i':
				info();
				break;
			case 't':
				temperature();
				break;
			case '0':
			case '1':
			case '2':
			case '3':
			case '4':
			case '5':
			case '6':
			case '7':
			case '8':
			case '9':
				x = r - '0';
				if (command == 'r') {
					cr = x;
				}
				if (command == 'g') {
					cg = x;
				}
				if (command == 'b') {
					cb = x;
				}
				command = 0;
				break;
			default:
				command = 0;
			}
		}

		leds();
	}
}
